[[_TOC_]]

# Email Virtual Domain application

## Other Documentation

Start by reviewing the information on these pages:

* [Email Virtual Domain service page][1] (client-facing).

* [Email Virtual Domain ikiwiki page][2].

The Email Virtual Domain web application allows the editing of members of
_existing_ e-mail virtual domains; for the creation of a _new_ e-mail
virtual domain see section "Other Documentation".

The updates made through this web interface are stored in a database, but
more importantly, are propagated to the LDAP; see the section
["Database Schema"](#database-schema) below for more detail on how the data
is synced into the LDAP.

## Glossary

* `container`: this is the email virtual domain that is used in e-mail addresses. Examples:
`idg.stanford.edu`, `law.stanford.edu`.

* `sub-domains`: a subdomain of a container.

* `alias`: A container contains a list of e-mail aliases.  An alias
contains one or more _forwarding addresses_.  An alias is used when
sending e-mail. E-mail sent to this address would be forwarded to the
aliases forwarding addresses.

* `forwarding address`: These are e-mail addresses that mail sent to an
_alias_ are delivered to.

**Example:** The container `idg` has three aliases with the following forwarding addresses:
```
adamhl:
  - adamhl@stanford.edu
  - adamh456@lewenberg.com
whm:
  - whm@stanford.edu
team:
  - adamhl@stanford.edu
  - whm@stanford.edu
  - rra@stanford.edu
```
Mail sent to `team@idg.stanford.edu` will be forwarded to
`adamhl@stanford.edu`, `whm@stanford.edu`, and `rra@stanford.edu`.

## Workflow

1. _Create new domain_: A customer (i.e., Stanford person) wants to create
   a new Email Virtual Domain. That process is _not_ handled by the web
   application; see the [Email Virtual Domain ikiwiki page][2] for
   instructions on how to create a new Email Virtual Domain.

1. _Add/Remove/Update aliases_: The customer wants to add, remove, or
   change aliases. They use the web interface provided by the software in
   this application to make the changes.

   When the changes are done and the user clicks on "Finish" the changes
   are saved to the back-end database and propagated to the LDAP master
   using the `ldap-alias-maint` script.

1. Delete domain: This is handled manually via a HelpSU ticket.


## Credentials

The web application both reads and writes to the LDAP: it reads when
displaying the containers and their aliases, and writes when a user
presses the "Finish" button after making changes. In order to read and
write, the application and supporting scripts need to know which LDAP
servers to connect to and be pointed to Kerberos credentials to
successfully bind to the LDAP servers.

These Kerberos credentials need to have the following
access in LDAP:
```
      cn=people,dc=stanford,dc=edu  READ    ldap:read_host
cn=applications,dc=stanford,dc=edu  READ    ldap:write_host
cn=applications,dc=stanford,dc=edu  WRITE   ldap:write_host
```

The application also needs to be able to read and write to the MySQL
database (the application's "source of truth"). The credentials to do this
are a username and password described in the "Database" subsection of
"Configuration" below.

## Configuration

The Email Virtual Domain application reads its configuration
from the YAML file `/etc/vemail/config.yaml`. This section
describes the configuration directives used in this YAML file.

### Kerberos settings

* `kerberos:keytab`: The full path to a Kerberos keytab file containing
credentials that have *both* read and write access to the LDAP database
(see also ["Credentials"](#credentials)).
Default: `/etc/vemail/keytab`.

* `kerberos:tgt`: When connecting to the LDAP the `kerberos:keytab`
file is used to generate a ticket cache. The `kerberos:tgt` configuration
directive is where this Kerberos ticket cache should be stored.
Default: `/var/run/web/tools.k5.tgt`.

### LDAP settings

* `ldap:read_host`: the LDAP endpoint to use when _reading_ Stanford
Workgroup information from LDAP. The credentials provided by the Kerberos
keytab must be able to read the `cn=people,dc=stanford,dc=edu` tree on
`ldap:read_host`.
Default: `ldap-dev.stanford.edu`.

* `ldap:write_host`: the LDAP endpoint to use when _writing_ to the LDAP.
Although this LDAP endpoint _is_ written to, the credentials provided by the Kerberos
keytab must be able to write to *and* read from the
`cn=applications,dc=stanford,dc=edu` tree on `ldap:write_host`.
Default: `ldap-dev-master.stanford.edu`.

* `ldap:wg_base`: The LDAP tree to use when reading Stanford Workgroup
information. This is used when deciding which containers a user
accessing the Email Virtual Domain application is allowed to manage.
Default: `cn=people,dc=stanford,dc=edu`.

### Database settings

These are the usual MySQL database connection settings.

* `db:host`: The hostname of the MySQL server.
Default: `127.0.0.1`

* `db:port`: The host port to use when connecting.
Default: `3306`.

* `db:database`: The name of the database containing the
application. Default: `s_email_virtual_dev.

* `db:username`: The username to use when authenticating to the
database server.
Default: `s_email_virtual_dev.

* `db:password_file`: The full path to the file containing the password to
use when authenticating to the database server.  This file should be a
single line consisting of the password.  The value of `db:password_file`
does not, under normal circumstances, need to be changed.
Default: `/etc/vemail/db-password`.


### Access Control

The `admins` directive is a list of Stanford Workgroups whose members
are administrators for the Email Virtual Domain application, that is,
these admins can read and make updates (including deleting) to any
mail alias in the application.

The default for this directive is:
```
admins:
  - helpdesk:techsupport
  - itservices:idg
```

### LDAP-to-Database mappings

The following are mappings of LDAP attribute names to their corresponding
names in the `containers` or `aliases` tables in the database. These values
should be left at their defaults:

####  LDAP-to-Database mappings (`containers` table)

The name of the field in the `containers` database table corresponding to the
specified LDAP attribute.

Default:
```
container_attr_mapping:
  cn:          container
  description: workgroup
```

####  LDAP-to-Database mappings (`aliases` table)

The name of the field in the `aliases` database table corresponding to the
specified LDAP attribute.

Default:
```
alias_attr_mapping:
  cn:              container
  mail:            mail
  sumaildrop:      maildrop
  suadministrator: owner
  description:     description
  owner:           IGNORE
```

### Miscellaneous

The following are other configuration directives that under normal circumstances
should not be changed.

* `template`: The HTML::Template template file name. Should be in the same directory
as the main index.cgi file. There should be no reason to change this value.
Default:`modern.tmpl`.

## Database Schema

### `containers`

```
+------------+-------------------------+------+-----+---------+-------+
| Field      | Type                    | Null | Key | Default | Extra |
+------------+-------------------------+------+-----+---------+-------+
| container  | varchar(128)            | NO   | PRI | NULL    |       |
| workgroup  | varchar(256)            | NO   |     | NULL    |       |
| contact    | varchar(256)            | NO   |     | NULL    |       |
| finance    | varchar(64)             | NO   |     | NULL    |       |
| status     | enum('active','frozen') | NO   |     | NULL    |       |
| dns_domain | varchar(255)            | YES  |     | NULL    |       |
+------------+-------------------------+------+-----+---------+-------+
```

Each entry in `containers` corresponds to an entry in the LDAP of the form
`cn=<container>,cn=applications,dc=stanford,dc=edu`.

The `status` field is only used by the web application: only containers with
`status` set to `active` will be displayed in the web application. In particular,
the `status` field does not get propagated to the LDAP.

### `aliases`

```
+-------------+---------------+------+-----+---------+-------+
| Field       | Type          | Null | Key | Default | Extra |
+-------------+---------------+------+-----+---------+-------+
| mail        | varchar(256)  | NO   | PRI | NULL    |       |
| maildrop    | varchar(512)  | NO   |     | NULL    |       |
| container   | varchar(128)  | NO   | PRI | NULL    |       |
| owner       | varchar(8)    | YES  |     | NULL    |       |
| description | varchar(1024) | YES  |     | NULL    |       |
+-------------+---------------+------+-----+---------+-------+
```

Each entry in `aliases` corresponds to an entry in the LDAP of the form
`cn=<mail>,cn=<container>,cn=applications,dc=stanford,dc=edu`. The `maildrop` field
is a comma-delimited list of e-mail addresses that are expanded into the LDAP
multi-valued attribute `suMailDrop`.

Example: the record in `aliases`
```
+---------+------------------------------------------------+-----------+---------+-----------------------------------------------+
| mail    | maildrop                                       | container | owner   | description                                   |
+---------+------------------------------------------------+-----------+---------+-----------------------------------------------+
| buddy3  | frankie4@lewenberg.com,frankie44@lewenberg.com | test      | adamhl  | asdasdasd                                     |
+---------+------------------------------------------------+-----------+---------+-----------------------------------------------+
```
corresponds to the LDAP entry
```
dn: cn=buddy3,cn=test,cn=applications,dc=stanford,dc=edu
objectClass: suMailRoute
suAdministrator: adamhl
cn: buddy3
mail: buddy3
description: asdasdasd
suMailDrop: frankie4@lewenberg.com
suMailDrop: frankie44@lewenberg.com
```

### `domains`

```
+-----------+--------------+------+-----+---------+-------+
| Field     | Type         | Null | Key | Default | Extra |
+-----------+--------------+------+-----+---------+-------+
| container | varchar(256) | NO   | PRI |         |       |
| domain    | varchar(256) | NO   | PRI |         |       |
+-----------+--------------+------+-----+---------+-------+
```


### Complete example of database/LDAP data correlation

The virtual email domain `test.stanford.edu` has five aliases.

In LDAP the container looks like this:
```
dn: cn=test,cn=applications,dc=stanford,dc=edu
objectClass: organizationalRole
cn: test
description: Container for itservices:idg

dn: cn=billy,cn=test,cn=applications,dc=stanford,dc=edu
objectClass: suMailRoute
cn: billy
mail: billy
suAdministrator: whm
suMailDrop: adamhl@lewenberg.com
description: California Zephyr, gonne but no forgotten

dn: cn=buddy2,cn=test,cn=applications,dc=stanford,dc=edu
mail: buddy2
cn: buddy2
suMailDrop: buddy4@macrotex.net
suMailDrop: buddy6@macrotex.net
description: Hello
objectClass: suMailRoute
suAdministrator: whm

dn: cn=psr123,cn=test,cn=applications,dc=stanford,dc=edu
description: psr123
mail: psr123
cn: psr123
objectClass: suMailRoute
suMailDrop: psr123@stanford.edu
suAdministrator: psr123

dn: cn=whmiii,cn=test,cn=applications,dc=stanford,dc=edu
objectClass: suMailRoute
suMailDrop: bill@ca-zephyr.org
cn: whmiii
mail: whmiii
description: California Zephyr, gone but no forgotten
suAdministrator: whm

dn: cn=jcowart,cn=test,cn=applications,dc=stanford,dc=edu
description: testing
suAdministrator: jcowart
cn: jcowart
objectClass: suMailRoute
mail: jcowart
suMailDrop: jcowart7777@gmail.com
```

In the database this looks like this:
```
SELECT * FROM container where container = 'test';
+-----------+----------------+--------------------+-----------------+--------+------------+
| container | workgroup      | contact            | finance         | status | dns_domain |
+-----------+----------------+--------------------+-----------------+--------+------------+
| test      | itservices:idg | sfeng@stanford.edu | 1025908-1-AABNM | active | NULL       |
+-----------+----------------+--------------------+-----------------+--------+------------+

SELECT * FROM aliases where container = 'test';
+---------+-----------------------------------------+-----------+---------+-------------------------------------------+
| mail    | maildrop                                | container | owner   | description                               |
+---------+-----------------------------------------+-----------+---------+-------------------------------------------+
| billy   | adamhl@lewenberg.com                    | test      | whm     | California Zephyr, gonne but no forgotten |
| buddy2  | buddy4@macrotex.net,buddy6@macrotex.net | test      | whm     | Hello                                     |
| jcowart | jcowart7777@gmail.com                   | test      | jcowart | testing                                   |
| psr123  | psr123@stanford.edu                     | test      | psr123  | psr123                                    |
| whmiii  | bill@ca-zephyr.org                      | test      | whm     | California Zephyr, gone but no forgotten  |
+---------+-----------------------------------------+-----------+---------+-------------------------------------------+
```



## Utility scripts

### `usr/bin/ldap-alias-maint`

The [`ldap-alias-maint`](usr/bin/ldap-alias-maint) script does the
synchronization of the database and the LDAP. Rremember that the
synchronization only goes in on direction: *from* the database *to* the
LDAP.
For more information see [`ldap-alias-maint`'s man page](docs/ldap-alias-maint).

This script is called when the user clicks on the "Finish" button after
making changes in the web application.

### `usr/bin/vemail-metadata`

The [`vemail-metadata`](usr/bin/vemail-metadata) script manages the
container database entries including showing container details and adding
new containers.
For more information see [`vemail-metadata`'s man page](docs/vemail-metadata).

## Testing

Here are the steps for testing. Unless otherwise indicated, the steps
apply to both production and non-production environments. For a list
of the environments, see the ["Environments" section of the service page][3].

1. Add *new* container (virtual domain) `foo`. Do this by accessing the
environment hosting the application and using the `vemail-metadata`
script.

        $ vemail-metadata add foo -d foo.stanford.edu -w uit-et-eit:iedo-staff \
        -c uit-et-eit-iedo-bulk@lists.stanford.edu -f 1028760-1-AABOU -s active
        $ vemail-metadata show foo

1. The above command adds the information to the application database but
does _not_ sync the database to the LDAP. To trigger the sync you need to add at
least one mail alias. To do this, go to the web interface of the web
application, select your new container, and add an alias.  Be sure to
click "Finish" so that the LDAP gets synced.

1. Do an ldapsearch to verify that the LDAP was synced properly. Check
that the container itself is correct and that the aliases you added in the
previous step appear.

        $ ldapsearch -h ldap-uat.stanford.edu -b cn=foo,cn=applications,dc=stanford,dc=edu  (UAT)
        $ ldapsearch -h ldap.stanford.edu     -b cn=foo,cn=applications,dc=stanford,dc=edu  (PROD)

1. Add the new domain to the test SMTP server's Puppet configuration and
test by sending an e-mail directly through `smtp-test.stanford.edu`. SANJAY
TO ADD DETAILS.

1. [PRODUCTION ONLY] Add the new domain to Stanford's mail routing
infrastructure. The routing changes side of a new e-mail virtual domain
requires the participation of Jason Cowart who will take care of
this. When adding a new virtual e-mail domain, the part that involves
ProofPoint routing change requires a ticket with ProofPoint support and
takes a few business days for ProofPoint support to do their part.

1. [PRODUCTION ONLY] Send mail to one of the mailboxes in the container
and verify that you (eventually) receive it in the forwarded mailbox.

1. Change the status of the container to "frozen":

        $ vemail-metadata mod foo -s frozen

1. Verify that the container no longer appears in the web interface. Note
that container status does _not_ appear as an attribute in LDAP: it only
affects visibility in the web application.

1. Remove container. First, delete it from the application database:

        $ vemail-metadata del foo
        $ vemail-metadata show foo
        vemail-metadata: container foo does not exist ...

1. Now you can delete it from the LDAP. To do this use the "delete" action
of `ldap-alias-main`:

        $ ldap-alias-maint delete foo

1. Verify that the container and all of its mailboxes are removed from the
LDAP:

        $ ldapsearch -h ldap-uat.stanford.edu -b cn=foo,cn=applications,dc=stanford,dc=edu  (UAT)
        $ ldapsearch -h ldap.stanford.edu     -b cn=foo,cn=applications,dc=stanford,dc=edu  (PROD)

1. [PRODUCTION ONLY] Send mail to one of the mailboxes in the deleted container and verify that
you (eventually) receive a bounce notice.


[1]: https://uit.stanford.edu/emailcalendar/vemail

[2]: https://ikiwiki.stanford.edu/service/email/virtual-domain/

[3]: https://code.stanford.edu/et-iedo/cloud/kube-vemail/-/blob/master/service.md#environments
