# NAME

ldap-alias-maint -- Read the MySQL aliases table and sync with LDAP

# USAGE

**ldap-alias-maint** help

**ldap-alias-maint** manual

**ldap-alias-maint** \[options\] search \[_cid1,cid2,..._ \[_mbx1,mbx2,..._\]\]

**ldap-alias-maint** \[options\] verify \[_cid1,cid2,..._ \[_mbx1,mbx2,..._\]\]

**ldap-alias-maint** \[options\] update \[_cid1,cid2,..._ \[_mbx1,mbx2,..._\]\]

**ldap-alias-maint** \[options\] delete cid

# DESCRIPTION

This script email virtual domain aliases entries from a MySQL database
and/or creates LDAP directory entries with an object class of suMailRoute.

The attributes that are required for an alias entry are fixed.  They
are cn, mail, description, owner, sumaildrop, and administrator.  A
properties file can be used to mapping arbitrary table columns into
the fixed attribute names.

The distinguished name for an alias entry if formed from a combination
of the attr\_mail field and the attr\_cn field from the mysql table.
For example given the alias table entry of:

    mail: mrochek
    sumaildrop: mrochek@freed.org
    container: pmdf

and using the default field mapping the entry dn will be:

    cn=mrochek,cn=pmdf,cn=applications,dc=stanford,dc=edu

# ACTIONS

There are four possible actions: "search", "verify", "update", and "delete". All of
these actions (except "delete") take zero, one, or two arguments.

If one argument is given then this argument should be a comma-delimited list of
containers `cid1,cid2,...`. In this case the action will apply only to
those containers matching those containers in the list.
The equivalent SQL query conditional for this case is

    (container='cid1' OR container='cid2' OR ...)

If two arguments are given then the first should be a comma-delimited list
of containers `cid1,cid2,...` while the second should be a
comma-delimited list of mailboxes `mbx1,mbx2,...`. In this case only
those containers matching the containers in the list `cid1,cid2,...` and
mailboxes matching any of those in the list `mbx1,mbx2,...`. The SQL
wildcard character `%` can be used in the list `mbx1,mbx2,...`.
The equivalent SQL query conditional for this case is

    (container='cid1' OR container='cid2' OR ...) AND (mail='mbx1 OR mail='mbx2' OR ...)

Note that in the above if there are any `%`'s in the mbx string the "="
is replaced by "LIKE" for that mbx string.

If no arguments are supplied then the the option `--all` MUST be
given. In this case, the actions will operate on all containers and all
mail aliases. Providing no arguments and omitting the `--all` option
will result in an error.

See section EXAMPLES for more detail.

- **search**

    This action searches for the containers and mailboxes supplied as extra
    arguments and displays the details of the containers and mailboxes it
    finds.

- **update**

    The **update** action updates the LDAP from the database entries.

    The first step is that each container in `cid1,cid2,...` is checked
    against its entry in the LDAP and the attributes in the LDAP are updated
    to match the values in the in the database. If any containers are not in
    the LDAP then the LDAP entry is created. If `cid1,cid2,...` is NOT given
    and the `--all` options is provided,
    then ALL containers in the database are synced. Note that if a container
    is in the LDAP but has been removed from the database the entry in the
    LDAP is NOT removed; this sort of update must be handled manually.

    The second step is that each mailbox in `mbx1,mbx2,...` is checked
    against its entry in the LDAP and the LDAP attributes synced against the
    database values. If `mbx1,mbx2,...` is NOT given then ALL mailboxes in
    the database are synced.

    To sync the mailbox elements including REMOVING any mailbox entries in the
    LDAP that are no longer in the database the list `mbx1,mbx2,...` should
    be omitted.

- **verify**

    This action is very similar to the update action except rather than doing
    any updates it checks to see if any updates are necessary. It generates no
    output. If no updates are necessary it exits with 0, otherwise it exits
    with 1. To see more details printed to standard output on where the
    differences are, run using the --verbose option.

- **delete**

    The "delete" action is special. It is used for deleting a _single_
    container and all of its mailboxes out of the LDAP. The only valid
    syntax it accepts is a single container name. Note that a container
    can be deleted out of the LDAP _only_ if it has already been deleted
    out of the database; to delete a container out of the database
    use the vemail-metadata script.

- **help**

    Display a short help string

- **manual**

    Display this man page.

# EXAMPLES

To ensure that that container "biology" exists in the LDAP and sync all of its
mailboxes, including removing any that no longer exist in the database:

    $ ldap-alias-maint update biology

To ensure that that the containers "biology" and "english" exists in the
LDAP and sync all of their mailboxes including removing any that no longer
exist in the database:

    $ ldap-alias-maint update biology,english

To sync the mailbox "adamhl" in the container "biology" into LDAP:

    $ ldap-alias-maint update biology adamhl

To sync the mailboxes "adamhl" and "psr123" in the container "biology" into LDAP:

    $ ldap-alias-maint update biology adamhl,psr123

To sync the mailbox "adamhl" and all mailboxes starting "student\_" in the
container "biology" into LDAP:

    $ ldap-alias-maint update biology adamhl,student_%

To sync all containers and all their mailboxes including removing any mailboxes no
longer existing in the database:

    $ ldap-alias-maint update --all

The following will result in an error and a program abort:

    $ ldap-alias-maint update

To delete a container and all of its mailboxes, first delete it
from the database using vemail-metadata then delete it using the
delete action:

    $ vemail-metadata delete foo
    $ ldap-alias-maint delete foo

# OPTIONS

- **--config=/path/file**

    The configuration file.  The file format is YAML.  Comment lines start
    with a '#'.  If not specified then the file /etc/vemail/config.yaml is
    used.  Valid configuration properties are:

        Property                             Default
        -----------------------              -----------------------------------
        add:                                 1
        delete:                              1
        container_base:                      cn=applications,dc=stanford,dc=edu
        verbose:                             1
        debug:                               0
        use_default_mail_domain:             0
        #
        db:host:                             127.0.0.1
        db:database:                         s_email_virtual_dev
        db:port:                             3306
        db:username:                         s_email_virtual_dev
        db:password_file:                    /etc/vemail/db-password
        #
        ldap:host:                           ldap-dev.stanford.edu
        ldap:base:                           cn=people,dc=stanford,dc=edu
        #
        alias_table:                         aliases
        alias_attr_mapping:mail:             mail
        alias_attr_mapping:cn:               container
        alias_attr_mapping:description:      description
        alias_attr_mapping:owner:            owner
        alias_attr_mapping:sumaildrop:       maildrop
        alias_attr_mapping:administrator:    suadministrator
        #
        container_table:                     containers
        container_attr_mapping:cn:           container
        container_attr_mapping:description:  workgroup

    db:password\_file is the path to a file that contains the database
    password.

- **--all**

    Use this option to select all containers and all mailboxes. Cannot be
    used with the extra arguments `cid1,cid2,...` or `mbx1,mbx2,...`.

- **--dryrun**

    Use with **--debug** to see what the update would change without actually
    making the changes.  This option is ignored for any action other than
    "update".

- **--help**

    Print out usage.

- **--debug**

    Print debugging information.

# AUTHORS

Bill MacAllister <whm@stanford.edu>

Adam H. Lewenberg <adamhl@stanford.edu>
