[[_TOC_]]

# Virtual Domain for Email Tasks

## Requesting a new virtual email domain

1. The requested domain name cannot be an alias of another domain. (What
does this mean?)

1. The requester should submit a HelpSU ticket with the following data:

    * Domain: idg.stanford.edu
    * Secondary domains: infrastructure-delivery-group.stanford.edu,
                         itservices-idg.stanford.edu
    * Management workgroup: itservices:idg
    * Contact email: its-idg-team@lists.stanford.edu
    * Oracle Financial Account: xxx-xxx-xxx

1. In the above example, emails to `foo@itservices-idg.stanford.edu` will
be treated the same as foo@idg.stanford.edu.. Any subsequent changes to
this metadata will have to be requested via the HelpSU interface.

1. Note: The Oracle Financial Account information is mainly there to
weed out frivolous requests from non-business groups and users since
there is no charge for this service.

1. Create the virtual domain:

        % remctl tools vemail add <domain> <options>

   For example, to create a virtual domain called *idg*:

        % remctl tools vemail add idg --workgroup='itservices:idg' \
          --contact=its-idg-team@lists.stanford.edu \
          --finance=<PTA> --domain=idg.stanford.edu

1. Notify the requester to go to the [Shared web application][1] to
   initialize the container.

   The domain admin should click on "Finish" button after adding new
   addresses in order to populdate the new addresses immmediately to the
   domain LDAP container. In case they forget to do this a daily (more
   often?) cron job will sync the data to LDAP for all virtual domains.

1. The LDAP container will be created after the first address is added. To check this:

        % remctl ldap-master ldapalias search <domain>


1. Test the email routing setup, for example:

        % remctl ldap-master ldapalias search idg
        % remctl expn hd expn <someuser>@idg.stanford.edu

    Make sure the result matches what's in the domain setup.

1. The puppet changes should be done before the DNS record change below,
   within the same 1 hour window. For example, if switch over is 9:00 AM,
   do the puppet change before 9:00 AM, then the DNS record change. In
   theory, you can do puppet ahead of time, but sometime clients email
   forwarding misconfigurations can cause messages be delivered to
   Stanford when the users aren't ready, so it is better to do puppet
   change close to DNS change.

1. Make the change in Puppet for the SMTP servers.

    * Update `manifests/init.pp` to configure the specific va-domain-ldap.

    * Update `files/etc/postfix/va-domain` file in puppet to add the virtual
      domain.

    * Update `templates/va-domain-maps` in s_emailrouter class to add the
      va-domain-ldap look up map.

    * Push out the puppet change to all gateways for the above changes.

1. Make the change to the MX record. Coordinate with the domain owner to
   release their domain MX record, and update the netdb mx1, mx2, and mx3's
   record to accept email for this domain.


## Deleting virtual email domain

TO DO

[1]: https://shared-email.stanford.edu
