# NAME

vemail-metadata - virtual email domain metadata administration script

# SYNOPSIS

**vemail-metadata** \[**-h**\]

**vemail-metadata** list

**vemail-metadata** show _container_

**vemail-metadata** add _container_ \[_options_\]

**vemail-metadata** mod _container_ \[_options_\]

**vemail-metadata** del _container_

# DESCRIPTION

The vemail-metadata scripts provides an administrative interface to
manage virtual email domain metadata stored in the database. Each
virtual email domain is identified by a container (which corresponds
to an LDAP container). Each container must have the following metadata
fields: workgroup, contact, finance, status, and domains. See the
OPTIONS section for more information on each field.

Note that this script displays and updates email virtual domain
container information in the backend database. It does NOT read or
update the corresponding entries in the LDAP; for that use the
ldap-alias-maint script.

The `list` action lists the name of all containers.

The `show` action displays all the metadata for a container.

The `add` action creates a new container and adds its metadata to the
database.  This action requires all the options (excluding help) be
specified.

The `mod` action modifies the metadata for an existing container.  This
action requires at least one option (excluding help) be specified.  The
specified options will modified the corresponding metadata fields.  Other
metadata fields will remain unchanged.

The `del` action deletes a container and all of its metadata from the
database.

# OPTIONS

- **-h**, **--help**

    Print out this documentation (which is done simply by feeding the script to
    `perldoc -t`).

- **-w** _workgroup_, **--workgroup**=_workgroup_

    Specify the administrative workgroup that manages the aliases for this
    container.  The virtual email domains web application will only allow
    members of this workgroup to access the specified container.

- **-c** _contact_, **--contact**=_contact_

    Specify the administrative contact address (preferably a list address) for
    this container.

- **-f** _finance_, **--finance**=_finance_

    Specify the Oracle Financials account number for this container.  This
    metadata field is for record keeping purposes only and will not be used
    except for virtual domains with large number of aliases that may require
    additional email filtering software licenses.

- **-s** _status_, **--status**=_status_

    Specify the current status of the container.  The value of this option can
    only be **active** or **frozen**.  The web application will not allow
    access of frozen containers.

- **-d** _domain_, **--domain**=_domain_

    Specify the hosts for this container.  These hosts will be added as MX
    records to the campus email gateways.  This option can be used multiple
    times.  Modifying a container with this option will wipe out all previous
    hosts information for a container and replace it with the new hosts.

# EXAMPLES

List container names:

    vemail-metadata list

Create a new helpdesk container:

    vemail-metadata add helpdesk --workgroup=workgroup:helpdesk-owners
    --contact=its-helpdesk@lists.stanford.edu --finance xxx-xxx-xxxx
    --domain=helpdesk.stanford.edu --domain=its-helpdesk.stanford.edu

Modify the helpdesk container administrative contact address:

    vemail-metadata mod helpdesk --contact helpdesk@lists.stanford.edu

Modify the helpdesk container virtual domains (the old domains will be
deleted):

    vemail-metadata mod helpdesk --domain=hd.stanford.edu
    --domain=its-hd.stanford.edu

Delete the helpdesk container:

    vemail-metadata del helpdesk

# AUTHORS

Huaqing Zheng <morpheus@stanford.edu>
Adam H. Lewenberg <adamhl@stanford.edu>
